#!/usr/bin/gawk -f

# check_html_pub.gawk - warning report generator for check_html_pub.sh
# Copyright (C) step, 2023
# License GPL2

# usage: [env AWKPATH=this_dir] gawk -f this_file ( path1'.html.ERR )+
# exit status:
#	0 (all OK or some warnings)
#	1 (user error or internal error)
#	3 (some input files exceed mdview's capabilities)

# n    int   the n-th input section. /^filepath:/ is the section delimiter
# c    int   error count for the current section of the current file
# fc   int   total errors for the current file
# tc   int   total errors over all files
# w    int   warning count for the current section of the current file
# fw   int   total warnings for the current file
# tw   int   total warnings over all files

@include "shared.gawk"

BEGIN {
	tc = tw = 0
}

BEGINFILE {
	++n; fc = fw = c = w = 0
}

# End of section
match($0, /^_filepath:/) {
	filepath = substr($0, RLENGTH + 1)
	# end the previous section
	end_section(filepath, n, c, w)

	# start a new section
	++n; c = w = 0
	next
}

/^@ERROR@/ {
	++c
	print n ":error:" substr($0, 8)
}

/^@WARNING@/ {
	++w
	print n ":warning:" substr($0, 10)
}

function end_section(filepath, n, c, w) {
	fc += c; tc += c
	fw += w; tw += w

	printf "\033[7msection %d of '%s': ", n, pretty(filepath)
	if (c == 1)
		printf "1 error; "
	else if (c > 1)
		printf "%d errors - see above; ", c
	if (w == 1)
		printf "1 warning"
	else if (w > 1)
		printf "%d warnings - see above", w
	print "\033[0m"
}

ENDFILE {
	if (c > 0 || w > 0) {
		end_section(FILENAME, n, c, w)
	}

	printf "=== %s total errors: %s; total warnings: %s ===\n", pretty(FILENAME), fc, fw
}

END {
	if (tc > 0) # only errors matter
		exit 3
}
