#!/bin/sh

# check_html_pub.sh - check HTML publish folder for mdview limitations
# Copyright (C) step, 2023
# License GPL2

#
# Process all .html files in INPUT_DIR. Input files should be prepared with
# prep_html5.sh first. Input files will not be changed but error report files
# will be written to INPUT_DIR, which must be writable.
# -----------------------------------------------------------------------------
#   OUTPUT_DIR can be published as online FAQS but take care of *.ERR files!
# -----------------------------------------------------------------------------
#
# This script runs check_for_mdview.sh to verify the structure of input HTML
# documents, which once converted to markdown, can exceed the capabilities of
# our target markdown viewer (mdview, see quirks.sh). For instance, mdview
# 2023.04 does not support bullet lists with 3 or more levels, quote blocks and
# complex tables. The script will only report conditions for which I couldn't
# find work-arounds, and halts with an error report in this case.

usage () {
	echo "usage: ${0##*/} INPUT_DIR
exit status:
	0 (all OK or some warnings)
	1 (user error or internal error - exit immediately)
	3 (some input files exceed mdview's capabilities - exit after testing all files)"
}

report_errors () { # $1-err_dir $2-html_in_dir
	local err_dir html_in_dir p report ret
	err_dir="${1:?}" html_in_dir="${2:?}"
	exit_error=0 ret=0 check_for_mdview_runs=0

	# build arg list for `gawk`
	set -- "$err_dir/STOP_"*".ERR"

	case "$1" in ''|*"*"*) set -- ;; *)
		gawk -f "${0%.sh}.gawk" "$@"
		ret=$?
	esac

	if [ $ret -ne 0 ]; then
		ret=3
		printf "\n\033[7m%s\033[0m\n" "${0##*/}: exit with errors"
		printf "      \033[7m %s \033[0m\n" \
		"                               " \
		"you must clear the errors above" \
		"                               "
		echo "\
________________________________________________________________________________
\"Mdview's capabilities exceeded\" means that the input document is correct but
it includes tags and/or attributes that mdview cannot handle. Please restructure
the input document to work around mdview's limitations. Details of each issue
include the file name of the affected input document, and the xml2 paths that
lead to the HTML tags that need to be restructured in that document.
Details are available in .ERR files located next to the input files.
Before restarting this program delete all .ERR files.
________________________________________________________________________________\
"
	fi

	return $ret
}


#######################################################################
#                                main                                 #
#######################################################################


. "${0%/*}/shared.sh" || exit 1

in_dir="${1:?}"

die_unless_some_file "$in_dir" html

. "${0%/*}/quirks.sh" || die 1 "error while sourcing quirks configuration"

#######################################################################
#                       Run check_for_mdview.sh                       #
#######################################################################

# After prep_html5's pass HTML files are clean. However, they could still use
# tags and attributes that exceed mdview's capabilities.

check_for_mdview="${0%/*}/check_for_mdview.sh" # depends package xml2

# the limitation tests and their arguments
# note: always set _xml2=1 otherwise report_errors won't find errors. _xml2=0 is reserved.
set \
"list_item_level_gt $mdview_max_ulist_level _xml2=1 _filepath=1" \
"any_table_warning _xml2=1 _filepath=1" \
;

for limitation in "$@"; do
	errf="$in_dir/STOP_${limitation%% *}.ERR"
	: > "$errf" # below we will append to $errf
done

for p in "$in_dir/"*.html; do
	for limitation in "$@"; do
		errf="$in_dir/STOP_${limitation%% *}.ERR"

		# DEBUG FILTER - this is where you can select just some files
		#case $p in *aaa_test*.*|*Networking.*) :;; *) continue ;; esac

		check_for_mdview_runs=$(($check_for_mdview_runs + 1))
		# args: see check_for_mdview.sh --help
		"$check_for_mdview" "$p" $limitation >> "$errf"
		ret=$?
		case $ret in
			0) exit_error=1 ;;
			1) die 1 "check_for_mdview.sh: user error or internal error" ;;
		esac

	done
done

[ $check_for_mdview_runs -gt 0 ] || die 1 "error: your DEBUG FILTER patterns exclude all input files"

for limitation in "$@"; do
	errf="$in_dir/STOP_${limitation%% *}.ERR"
	if [ -e "$errf" ]; then
		[ -s "$errf" ] || rm -f "$errf"
	fi
done

report_errors "$in_dir" "$in_dir"

[ $exit_error -eq 1 ] && die 3 "error: some input documents exceed mdview's capabilities
restructure the documents and restart"

exit 0
