#!/usr/bin/gawk -f

# html5_to_ihtml.gawk - quirks engine for html5_to_ihtml.sh
# Copyright (C) step, 2023
# License GPL2

# usage: [env AWKPATH=this_dir] gawk -f this_file -v OUT_DIR=path -vQUIRK_NAME=VALUE... *.html
#

# -----------------------------------------------------------------------------
#   IMPORTANT: for this engine to work well, input HTML must be line-oriented
# -----------------------------------------------------------------------------

@include "shared.gawk"

BEGINFILE {
	outf = FILENAME
	sub(/.*\//, "", outf)
	outf = OUT_DIR "/" outf
}

{
	S = $0
	# tidy converts tags to lowercase
}

# --------------------------------------------------------------------------- #
### track <pre>

/<pre[^>]*>/   { IN_PRE = 1 }
/<\/pre>/      { IN_PRE = 0 }


# --------------------------------------------------------------------------- #

!IN_PRE {

	# a markdown header with Fatdog64's logo will be added elsewhere
	# delete the logo image typically nested inside tag <h1>.
	sub(/<img[ \t].*src="[^>]*fatdog.png"[^>]+>/, "", S)

	# prevent HTML <small>text</small> from spilling into mdview as
	# <span class="small">
	gsub(/<\/?small[ \t]*>/, "", S)
}

# --------------------------------------------------------------------------- #

### case 1: redirect links to local html files to their corresponding markdown file
# !!! assume that the markdown file exists !!!

# make some adjustments in passing:
# A1. Erase <br> of <a>/text() because newline in md link text is invalid.
# A2. wrap filepath's "/" of <a>/text() with code spans to avoid triggering
#     what seems to be a bug in 2023.04 (observed in faq.html#12)
# A3. erase <a> attrs except id,name otherwise pandoc will write attrs as
#      "<span>" inits markdown output

BEGIN {
	# priority order href,id,name ensured by `tidy --priority-attributes` (setup_tidy.sh)
	re_href = @/<a href="([^"]+)"( id="([^"]+)")?( name="([^"]+)")?([^>]*)>/

	# practical URI scheme + "://"
	re_uri_schemes = @/\<([a-z][-.+a-z0-9]*):\/\//

	# my opinionated take on a set of filepath characters in vanilla markdown content
	# square brackets excluded because usage in the text of [text](link) markdown is assumed
	re_filepath_md = @/\.?\/[!%&()+,-.\/:=@\\^_~[:alnum:]]+/

	debug_tag_a = 0
	debug_tag_a_breakpoint = 0
}
function get_tag_a(s, a,   pos) { # return array a {{{
	if (!match(s, re_href, a))
		return 0
	if (debug_tag_a) print FILENAME" "NR" "s > "/dev/stderr"
	a["a<"]        = substr(s, 1, RSTART - 1)
	a["href<"]     = "<a href=\""
	a["href"]      = a[1]
	a["href>"]     = "\""
	if (a[3]      != "") {
	a["id<"]       = " id=\""
	a["id"]        = a[3]
	a["id>"]       = "\""
	}
	if (a[5]      != "") {
	a["name<"]     = " name=\""
	a["name"]      = a[5]
	a["name>"]     = "\""
	}
	if (a[6]      != "") {
	a["attrs<.>"]  = a[6]
	}
	# FIXME this is the _html_ not the _text_ inside <a>, e.g. <a><em>text</em></a>, html == <em>text</em>
	a["text<"]     = ">"; s = substr(s, RSTART + RLENGTH)
	a["text"]      = substr(s, 1, (pos = index(s, "</a>")) - 1)
	a["text>"]     = "</a>"
	a["a>"]        = substr(s, pos + 4)
	if (debug_tag_a) for (k in a) printf "get_tag_a a[%s](%s)\n", k, a[k] > "/dev/stderr"
	return 1
} #}}}

BEGIN {
	mdview_can_jump_to_anchor = mdview_can_jump_to_anchor+0
}
#BEGINFILE { if (FILENAME ~ /boot-options\.html$/) debug_tag_a = 1 }
#ENDFILE { if (debug_tag_a_breakpoint) if (FILENAME ~ /boot-options\.html$/) exit 1 }

!IN_PRE && get_tag_a(S, a) {
	if (debug_tag_a) print "^^^^",FILENAME,NR > "/dev/stderr"
	done = todo = ""
	do {
		if (a["href"] !~ /^[[:alnum:]]+:\/\//) { # internal link
			if (match(a["href"], /\.html(#.*)?$/, segment)) {
				# rewrite .html extension as .md
				# if segment[1] => ?erase segment
				a["href"] = substr(a["href"], 1, RSTART - 1) ".md" (mdview_can_jump_to_anchor? segment[1] : "")
			}
			else {
				# either url links to a document section or to
				# a local file name that doesn't end with ".html"
			}
		}
		else {
			# url links to an external site
		}

		#if (debug_tag_a) printf "A3: a[text](%s)\n", a["text"] > "/dev/stderr"
		if ((text = a["text"]) != "") {
			# A1. Erase <br> of <a>/text()
			gsub(/<br>/, "", text)

			# A2. wrap absolute filepaths in <a>/text() with <code> span
			# unless the whole text includes a URI scheme
			buf = ""
			if (debug_tag_a) printf "start text(%s)\n", text > "/dev/stderr"
			if (text !~ re_uri_schemes) while (match(text, re_filepath_md)) {
				# catch false positives
				if ((RSTART == 1 && substr(text, RSTART, 1) != "/") || (RSTART > 1 && substr(text, RSTART - 1, 2) !~ /[ \t]\//)) {
					RSTART += RLENGTH + 1 # jump over this false positive
					RLENGTH = 0
					if (debug_tag_a) printf "FALSE POSITIVE " > "/dev/stderr"
				}
				if (debug_tag_a) printf "matched(%s)\n", substr(text, RSTART, RLENGTH) > "/dev/stderr"
				buf = (buf substr(text, 1, RSTART - 1) (RLENGTH > 0 ? "<code>" :"") substr(text, RSTART, RLENGTH) (RLENGTH > 0 ? "</code>" : ""))
				if (debug_tag_a) printf "buf(%s)\n", buf > "/dev/stderr"
				text = substr(text, RSTART + RLENGTH)
				if (debug_tag_a) printf "next text(%s)\n", text > "/dev/stderr"
			}
			a["text"] = (buf text)
			if (debug_tag_a) printf "A3 end text: a[text](%s)\n", a["text"] > "/dev/stderr"
		}

		if (debug_tag_a) print FILENAME, NR, S > "/dev/stderr"
		if (debug_tag_a) for (k in a) if (k !~ /[0-9]/) printf "A3: a[%s](%s)\n", k, a[k] > "/dev/stderr"

		# A3: omit other attributes
		done = done (a["a<"] a["href<"] a["href"] a["href>"] a["id<.>"] a["name<.>"] a["text<"] a["text"] a["text>"])
		todo = a["a>"]
		if (debug_tag_a) printf "done(%s)\ntodo(%s)\n", done, todo
	} while (get_tag_a(todo, a))
	S = (done todo)
	if (debug_tag_a) print FILENAME" "NR" S("S")" > "/dev/stderr"
}

# --------------------------------------------------------------------------- #

### case 2: Replacing <ol> with <ul>
# if mdview can't support ordered list replace <ol> with unordered list <ul>
# and insert bold item counter in contained <li>

BEGIN {
	ol2ul = mdview_max_olist_level+0 < 0
}
BEGINFILE {
	olul_top = 0
	delete li_cnt
}
# match list open/close tag <ol> </ol> <ul> </ul>
ol2ul && match(S, /^(<\/?)([ou]l)([ \t]?[^>]*>)/, a) {
	# pop/push ol/ul
	if (a[1] == "</") {
		delete li_cnt[olul_top]
		--olul_top
	} else {
		olul[++olul_top] = a[2]
		li_cnt[olul_top] = 0
	}
	# ol => ul
	if (a[2] == "ol") {
		S = a[1] "ul" a[3] substr(S, RSTART + RLENGTH)
	}
}
# match list item open tag <li>
ol2ul && olul[olul_top] == "ol" && match(S, /^(<li[ \t]?[^>]*>)(.*)$/, a) {
	# insert counter
	S = a[1] "<b class=\"ihtml\">"get_incr_li_cnt(olul_top, 2)"</b> " a[2]
}
function get_incr_li_cnt(top, fmt_type,   sep, i, cnt, ret) { # SIDE EFFECT

	# if fmt_type == 1: simple counter n
	ret = ++li_cnt[top]

	if (fmt_type == 2) { # nested i.j.k counter
		for (i = 1; i < top; i++) {
			if (olul[i] == "ol") {
				cnt = cnt sep li_cnt[i]
			}
		sep = "."
		}
		ret = cnt sep ret "."
	}

	return ret
}


# --------------------------------------------------------------------------- #

### Pass ihtml_to_pub{,_pre}.gawk, aka PUB, hints to deindent hard-wrapped list items

# First idea: hide <br> to pandoc. Later we will convert them into line breaks.
# Since pandoc can't see the <br> anymore, and --wrap=0 is set, the list
# item outputs as and remains a whole line.
# Second idea: mark </ul> and </ol> because they're opportunities for pandoc
# to introduce hard-line breaks hence to hard-wrap the list item. PUB{,_pre}
# will deindent paragraphs at and after the mark.

!IN_PRE {
	# PUB squeeze white-spaced <br>s to a \n
	gsub(/<br>/, "%IHTMLbr%", S)

	# PUB deindent until next list item
	gsub(/<\/[ou]l>/, "&<p class=\"ihtml\">%IHTMLdeindent%</p>", S)
}

# --------------------------------------------------------------------------- #

### Work-around for <li>"-"DIGIT

# With pandoc "<li>-32" becomes "    -    -32" - blockcode for mdview

!IN_PRE {
	if (match(S, /<li>-([0-9]+)/, a))
		S = (substr(S, 1, RSTART - 1) "<li>&nbsp;-"a[1] substr(S, RSTART + RLENGTH))
}


# --------------------------------------------------------------------------- #

{
	print S > outf
}

ENDFILE {
	close(outf)
}

function die_tag_not_closed(tag) {
	printf "%s: error: %s line %s: in-line closing tag missing for %s\n", SN, pretty(FILENAME), FNR, tag > "/dev/stderr"
	exit 1
}
