#!/usr/bin/gawk -f

# ihtml_to_md.gawk - post-process pandoc markdown
# Copyright (C) step, 2023
# License GPL2

# usage: [env AWKPATH=this_dir] gawk -f this_file -vQUIRK_NAME=VALUE... [CURRENT_FILE=path]   (stdin/stdout filter)
# stdin is pandoc markdown_strict
# stdout is mdview markdown

#
# This script adapts pandoc markdown to mdview-friendly markdown.
#

@include "shared.gawk"

FNR == 1 {
	fd_header()
}

{
	S = $0
}


# --------------------------------------------------------------------------- #

### Unindent fenced code block marks.

# Pandoc indents fenced code block marks when the code block is included in a
# bullet item.

BEGIN {
	re_indented_fenced_codeblock_mark = @/^ {4,}```/
}

S ~ re_indented_fenced_codeblock_mark { S = "```" }


# --------------------------------------------------------------------------- #

### track codeblock

BEGIN {
	in_fenced_codeblock = 0
	in_codeblock = 0

	re_fenced_codeblock = @/^```/

	# /^ {n}[^- ]/ is for list bullets; see the next "###" block
	re_codeblock = @/^\t|^ {4}[^- ]|^ {8}[^- ]|^ {12}[^- ]|^ {16}[^- ]|^ {20}[^- ]|^ {24}[^- ]|^ {28}[^- ]|^ {32}[^- ]/

	re_not_codeblock = @/^ {0,3}[^\t ]/
}

function FCB_START(s)  { return !in_fenced_codeblock && s!="" && s ~ re_fenced_codeblock }
function FCB_END(s)    { return  in_fenced_codeblock && s!="" && s ~ re_fenced_codeblock }
function  CB_START(s)  { return !in_fenced_codeblock && !in_codeblock && s!="" && s ~ re_codeblock }
function  CB_END(s)    { return         in_codeblock && s!="" && s ~ re_not_codeblock }
function  IN_CBS()     { return  in_fenced_codeblock || in_codeblock }

{
	if      (FCB_START(S))   { in_fenced_codeblock = 1 }
	else if (FCB_END(S))     { in_fenced_codeblock = 0 }
	else if (CB_START(S))           { in_codeblock = 1 }
	else if (CB_END(S))             { in_codeblock = 0 }
}

#IN_CBS() { print "IN_CBS", in_fenced_codeblock && !in_codeblock ? "fenced" : (in_codeblock && !in_fenced_codeblock ? "regular" : "ERROR") > "/dev/stderr" }


# --------------------------------------------------------------------------- #

### replace well-known HTML character entities

# Normally pandoc replaces numeric character entities with the corresponding
# Unicode character. However, for some characters or numeric entities it
# outputs their HTML entity, which mdview renders verbatim. Therefore we
# replace known HTML entities here.

!IN_CBS() {
	gsub(/&num;/, "#", S)   # 35
	gsub(/&amp;/, "\\&", S) # 38
	gsub(/&lt;/, "<", S)    # 60
	gsub(/&gt;/, ">", S)    # 62
}


# --------------------------------------------------------------------------- #

### Get rid of <span> tags

# There are cases in which pandoc writes a <span> to markdown, for instance
# when the span includes an id attribute. Mdview markdown has no use for
# HTML tags.

BEGIN {
	re_tag_span = @/<\/?span[^>]*>/
}

!IN_CBS() && S ~ re_tag_span {
	gsub(re_tag_span, "", S)
}


# --------------------------------------------------------------------------- #

### Time to resolve %IHTMLdeindent% hint
# (a work-around for hard-wrapped list items)

# To actually de-indent we erase the mark and de-indent to column-1 every
# subsequent line until we hit a line that starts in column 1 or a bullet.
# If we find another %IHTMLdeindent% mark on our way we simply erase it.
# We do all this regardless of IN_CBS() state.
# ihtml_to_md_pre.gawk

BEGIN {
	in_IHTMLdeindent = 0

	re_IHTMLdeindent = @/%IHTMLdeindent%/
	# also re-using re_4space_bullet for the bullet match
}

# exit de-intenting mode
in_IHTMLdeindent && S !~ re_IHTMLdeindent && (S ~ re_4space_bullet || S ~ /^\S/) {
	in_IHTMLdeindent = 0
}

# de-indent line to column 1 while in de-indenting mode
in_IHTMLdeindent {
	sub(/^ +/, "", S)
}

# enter or re-enter de-intenting mode
match(S, re_IHTMLdeindent) {
	in_IHTMLdeindent = 1

	# erase the mark but leave what follows it, even if it's white space
	S = substr(S, RLENGTH + 1)
}


# --------------------------------------------------------------------------- #

### work around pandoc's fixation with 4-space indentation stops for list items

# Running AST/BulletList.sh shows the following specimen of pandoc's
# 4-space indentation rule in action:
#
#-   -   -   -   -   5
#
#<!-- -->
#
#-   1
#
#-   -   2
#
#    -   -   3
#
#        -   -   4
#
#            -   -   5
#
# To convert to faux mdview bullets we need to replace the space in
# every 4-space tab stop above with a dash until we hit item content.

BEGIN {
	# regex covers the speciment and other observed cases:
	re_4space_bullet = @/^([- ] {3}){1,}[^- ]/
}
function GET_BULLETS(s,   b) {
	if (match(s, re_4space_bullet) && index(b = substr(s, RSTART, RLENGTH -1), "-"))
		return b
	return ""
}

!IN_CBS() && bullets = GET_BULLETS(S) {
	while (sub(/ {4}-/, "-   -", bullets))
		;
	S = (bullets substr(S, RLENGTH))
}


# --------------------------------------------------------------------------- #

### translate UTF-8 smilies to ASCII
# not needed anymore: pandoc replaces &# entities with the actual character.

# !IN_CBS() {
# 	gsub("&#9786;", ":)", S) # faqs/bugs.html
# }


# --------------------------------------------------------------------------- #

### if mdview can't treat /  $/ as a hard line break, replace /  $/ with \n

BEGIN {
	mdview_can_br_two_space_suffix = mdview_can_br_two_space_suffix+0
}
!mdview_can_br_two_space_suffix && !IN_CBS() {
	gsub(/  $/, "\n", S)
}


# --------------------------------------------------------------------------- #

### done

{
	print S
}

ENDFILE {
	fd_footer()
}
