#!/bin/sh

# ihtml_to_md.sh - convert HTML to markdown
# Copyright (C) step, 2023
# License GPL2

#
# Convert all .html files in INPUT_DIR and write results to OUTPUT_DIR.
# INPUT_DIR holds "normalized", intermediate HTML (see ./html5_to_ihtml.sh).
#
# This script massages HTML to work around limitations of mdview markdown viewer
# and stylistic differences between HTML and mdview's GTK2 output. Ultimately
# we aim to output mdview-friendly markdown.

# MDVIEW TRANSLATION SUPPORT
# By default ./quirks.sh enables mdview's gettext support.
# See /usr/share/mdview/examples/gettext.md.
# If $mdview_enable_gettext_support == 1 then `class="mdview-nopot"` in HTML block
# elements becomes a %%nopot wrapper in markdown.
#
# Note: **Editors** can add `class="mdview-nopot"` to the block element tags that
# shouldn't be translated in mdview. `<pre>` elements are not translated by
# default unless they include `class="mdview-gettext"`.


usage () {
	echo "usage: [DEBUG=1] ${0##*/} INPUT_DIR OUTPUT_DIR
exit status:
	0  (all OK)
	>0 (error)
DEBUG=1:
	Don't create file index faq_file_list.md.
	Create markdown_strict (.mkd) file alongside mdview markdown (.md) file."
}

### main

. "${0%/*}/shared.sh" || exit 1

die_unless_in_dir_out_dir "$@" # => $in_dir $out_dir

#. "${0%.sh}_bbtest.sh" || die 1 "error while sourcing black-box tests"

die_unless_some_file "$in_dir" html

. "${0%/*}/quirks.sh" || die 1 "error while sourcing quirks configuration"

. "${0%/*}/setup_pandoc.sh" || die 1 "error while setting up pandoc command"

### main

# FAQ file list  - link in shared.gawk:fd_footer()
if [ -z "$DEBUG" ]; then
	file_list="$in_dir/faq_file_list.html"
	exec 3> "$file_list"
	cat >&3 << EOF
$(gawk '/DOCTYPE/,/<h1>/' "$in_dir/faq.html")
<hr>
File list.<br>
<ul>
EOF
fi

# create argument list and file list
set --
for p in "$in_dir/"*.html; do
	# don't need markdown for this file
	case $p in *faq_outside.*) continue ;; esac

	# DEBUG FILTER - this is where you can select just some files
	#case $p in *aaa_test*.*|*Networking.*) :;; *) continue ;; esac

	set -- "$@" "$p"
	f="${p##*/}"; f="${f%.html}"
	[ -z "$DEBUG" ] && echo "<li><a href=\"$f.md\">$f</a></li>" >&3
done
if [ -z "$DEBUG" ]; then
	echo >&3 "\
</ul>
</div>
</body>
</html>"
	exec 3>&-
fi

counter=0
[ 1 -eq $((${DEBUG:-0} && $# > 0 || !${DEBUG:-0} && $# > 1)) ] && \
for p; do                         # ^^^^^^^^^^^^ making $file_list
	printf .
	counter=$(($counter + 1))
	mdf="$out_dir/${p##*/}"
	mdf="${mdf%.*}.md"           # mdview markdown (publish)
	# pandoc markdown_strict
	[ "$DEBUG" ] && mkdf="${mdf%.*}.mkd" || mkdf='/dev/null'

	pandoc $pandoc_options $lua_filters \
		--from html$from_extensions \
		--to   $mdview_markdown$to_extensions \
		"$p" |
		tee "$mkdf" | # debug aid

	gawk -f "${0%.sh}_pre.gawk" |

	gawk -f "${0%.sh}.gawk" $QUIRKS_AWK_V CURRENT_FILE="$p" > "$mdf" ||
		die 1 "error converting '${p##*/}' to markdown"

	[ -s "$mdf" ] || rm "$mdf"
done
echo

[ $counter -gt 0 ] || die 1 "error: your DEBUG FILTER patterns exclude all input files"

