#!/usr/bin/gawk -f

# prep_html5.gawk - report generator for prep_html5.sh
# Copyright (C) step, 2023
# License GPL2

# usage: [env AWKPATH=this_dir] gawk -f this_file ( HTML_FILE=path1.html path1'.html.ERR )+
# exit status: 0 but possibly with warnings and errors on stdout

# n    int   the n-th input file
# c    int   combined warning count for the current .html and .html.ERR files
# e    int   error count for the current .html.ERR file
# W[n] str   warnings raised by this awk script - "\n" separated
# C[n] str   c:n key:value, implemented as c" "n
# F[n] str   FILENAME
# H[n] str   HTML_FILE
# tc   int   total warnings

@include "shared.gawk"

BEGINFILE {
	die_if_mismatched_args(HTML_FILE, FILENAME)

	++n; c = e = 0

	H[n] = HTML_FILE

	# case 1: want HTML5
	if (0 < (getline s < HTML_FILE) && close(HTML_FILE) == 0) { die_read_error(HTML_FILE) }
	if (s != "" && s !~ /^<!DOCTYPE html>/) {
		# in tidy's format
		W[n] = "line 1 column 1 - Warning: document does not start with HTML5 doctype"
		++c
	}
}

# tidy has verbose output so we need to match
tolower($0) ~ /warning/ { ++c }
tolower($0) ~ /error/   { ++e }

ENDFILE   {
	if (c != 0) {
		tc += c
		C[n] = c" "n
		F[n] = FILENAME
	} else if (e == 0) {
		# disposable .ERR file
		error_free = error_free " '" FILENAME "'"
	}
}

function comp_func(i1, v1, i2, v2,   a, c1, c2) {
	split(v1, a); c1 = a[1] + 0
	split(v2, a); c2 = a[1] + 0
	return c1 > c2
}

END {
	if (tc) {

		m = asort(C, C, "comp_func")
		for (i = 1; i <= m; i++) {
			split(C[i], a)
			c = a[1] + 0
			n = a[2] + 0

			# warnings raised by this script
			wn = split(W[n], a, "\n")
			for (j = 1; j <= wn; j++) {
				print n":"a[j]
			}

			# like `sed s/^/:<n>/ FILENAME`
			ef = F[n]
			while (0 < (getline s < ef)) {
				print n":"s
			}
			close(ef)

			fmt = dcngettext(_"1 warning", _"%d warnings", c)
			printf "\033[7mfile "n"\033[0m:" "%s \033[7m" fmt "\033[0m see above\n", pretty(H[n]), c
		}
		printf "=== total warnings: %d ===\n", tc
	}

	if (error_free != "") { system("rm -f "error_free) }

	exit 0
}

# validate basename(hf) == basename(errf, ".ERR")
function die_if_mismatched_args(hf, errf) {
	sub(/.*\//, "", hf)
	sub(/.*\//, "", errf); sub(/\.ERR$/, "", errf)
	if (hf errf == "" || hf != errf) {
		printf "%s: error: %s line %s: file names don't match '%s' <=> '%s'\n", SN, pretty(FILENAME), FNR, hf, errf > "/dev/stderr"
		tc = 0
		exit 3
	}
}

function die_read_error(f) {
	printf "%s: error: %s line %s:  empty file or error reading '%s'\n", SN, pretty(FILENAME), FNR, hf, f > "/dev/stderr"
}
