#!/bin/sh

# prep_html5.sh - prepare HTML for further processing
# Copyright (C) step, 2023
# License GPL2

#
# Process all .html files in INPUT_DIR and write results to OUTPUT_DIR.
# Result is "normalized" HTML, in the sense that HTML output is ready for
# structural verification with check_html_pub.sh, and it is suitable for more
# specialized processing with html5_to_ihtml.sh before the final conversion to
# markdown with ihtml_to_md.sh.
# This script writes error/warning report files *.ERR to OUTPUT_DIR.
# -----------------------------------------------------------------------------
#   OUTPUT_DIR can be published as online FAQS but take care of *.ERR files!
# -----------------------------------------------------------------------------
#
# The 5 in prep_html5 is a reminder for content editors that I recommend to use
# HTML5 markup for all FAQs instead of older HTML versions. HTML5 source makes
# converting to markdown easier. Yet this script should be able to prep all
# versions of HTML. However, it's been mostly tested with HTML5 documents.
#
# This script starts as a bulk html `tidy` runner comprising calibrated options
# for `tidy` and a report generator, so that content editors can safely use this
# script in a write-test-fix workflow cycle - see README.md for details.
#
usage () {
	echo "usage: ${0##*/} INPUT_DIR OUTPUT_DIR
exit status:
	0 (all OK / tidy warned)
	1 (user error or internal error)
	2 (tidy found errors)"
}

report_errors () { # $1-err_dir
	local err_dir="${1:?}"
	awk '/Error:/ {print substr(FILENAME, 1, length(FILENAME) -4)  :"$0}' "$err_dir/"*".html.ERR"
}

report_warnings () { # $1-err_dir $2-html_in_dir
	local err_dir html_in_dir p report ret
	err_dir="${1:?}" html_in_dir="${2:?}" ret=0

	# build arg list for `gawk`
	set --
	for p in "$err_dir/"*.html.ERR; do
		# args: HTML_FILE=$html_in_dir/file.html $err_dir/file.html.ERR
		set -- "$@" "HTML_FILE=$html_in_dir/${p%.ERR}" "$p"
	done

	if ! report="$(gawk -f "${0%.sh}.gawk" "$@")"; then
		ret=3
	fi

	if [ -n "$report" ]; then
		printf "%s\n" "$report"
		printf "\n\033[7m%s\033[0m\n" "${0##*/}: reported warnings"
		printf "      \033[7m %s \033[0m\n" \
		"                                       " \
		"recommended to clear the warnings above" \
		"                                       "
	fi

	return $ret
}


#######################################################################
#                                main                                 #
#######################################################################


. "${0%/*}/shared.sh" || exit 1

die_unless_in_dir_out_dir "$@" # => $in_dir $out_dir

#. "${0%.sh}_bbtest.sh" || die 1 "error while sourcing black-box tests"

die_unless_some_file "$in_dir" html

. "${0%/*}/setup_tidy.sh" || die 1 "error while setting up tidy command"


#######################################################################
#                              Run tidy                               #
#######################################################################

counter=0

# build arg list for `tidy`
set --
for p in "$in_dir/"*.html; do
	# skip symlinks, like index.html -> faqs.html
	[ -L "$p" ] && continue

	# DEBUG FILTER - this is where you can select just some files
	#case $p in *aaa_test*.*|*Networking.*) :;; *) continue ;; esac

	counter=$(($counter + 1))
	# args: -o output_file -file error_report_file input_file
	set -- "$@" -o "$out_dir/${p##*/}" -file "$out_dir/${p##*/}.ERR" "$p"
done

[ $counter -gt 0 ] || die 1 "error: your DEBUG FILTER patterns exclude all input files"

LANG=EN_us.UTF-8 tidy $tidy_options "$@"

case $? in
	0|1)
		report_warnings "$out_dir" "$in_dir"
		case $? in
			3) die 1 "error: awk reported an error" ;;
		esac
		exit 0
		;;
	2)
		report_errors "$out_dir" "$in_dir"
		die 2 "error: tidy found errors; fix them and restart"
		;;
esac

