# This file is sourced, not run

# setup_pandoc.sh - set up pandoc run for ihtml_to_md.sh
# Copyright (C) step, 2023
# License GPL2

# https://pandoc.org/MANUAL.html
# https://pandoc.org/lua-filters.html
# https://pandoc.org/try online converter
# http://chulsky.com/pandoc/ AST, filters

# Configure:
# $pandoc_options $from_extensions $to_extensions $lua_filters

# quirks.sh
export mdview_markdown

# ----- PANDOC DEFAULTS---------------------------------------------------{{{
# Initialize from_extensions, to_extensions
. "${0%/*}/pandoc_defaults.sh"

# ------------------------------------------------------------------------}}}



# ----- PANDOC OPTIONS ---------------------------------------------------{{{
# Pandoc has so many options.
pandoc_options=""

# MARKDOWN LINE WRAPPING
# It is crucial to run pandoc with option --wrap=none, see point 3 under LISTS
# in this file.
#
# Recall that in .ihtml files each paragraph stretches over a single,
# potentially very long line, which we publish online.  Long lines aren't a
# problem for an HTML browser. However, for a human reader who reads markdown
# _with a text editor_ - even though we target mdview - long lines are a
# nuisance if the text editor can't wrap lines.
#
# --columns=N sets the wrapping width of paragraphs and table content to N
# characters. For instance, for N=80 and a 2-column table, table cells are
# limited to (80 - 7) / 2 characters (7 accounts for cell dividers and margins).
# --wrap=auto enables wrapping, --wrap=none disables it.
# =============================================================================
# Notes to Fatdog64 devs: translators will work on HTML source text - not on
# markdown - using tools from the fatdog-doctoo package.
# =============================================================================
pandoc_options="$pandoc_options --wrap=none"

# ATX-style (#-prefixed) or Setext-style (underlined) headings for level 1 and 2
# headings. ATX-style headings are always used for levels 3+.
# Mdview can render both kinds. ATX for uniformity.
pandoc_options="$pandoc_options --markdown-headings=atx"

# Preserve tabs instead of converting them to spaces. This only affects tabs in
# literal code spans and code blocks. Tabs in regular text are always treated as
# spaces.
pandoc_options="$pandoc_options --preserve-tabs --tab-stop=4"  # case 2

# When the --standalone option is used, pandoc uses a template to
# add header and footer material that is needed for a self-standing
# document. Run `pandoc -D $mdview_markdown` to see the default template.
# See also https://pandoc.org/MANUAL.html#templates. Option --template implies
# option --standalone.  Example:
#pandoc_options="$pandoc_options --template=${0%/*}/templates/fd-faq.$mdview_markdown"
#
# We don't use this feature because ihtml_to_md.gawk is our templating (and
# final fix-up) engine.
# ------------------------------------------------------------------------}}}



# ----- FROM EXTENSIONS --------------------------------------------------{{{
# in addition to $from_extensions in pandoc_defaults.sh
# ------------------------------------------------------------------------}}}



# ----- TO EXTENSIONS AND LUA FILTERS     --------------------------------{{{

#############################################################################
#                 >>> The order of LUA filters matters! <<<                 #
#############################################################################

# in addition to $to_extensions in pandoc_defaults.sh
lua_filters=""



### GENERAL

### TYPOGRAPHY ###
# no smart quotes and friends in markdown files FIXME don't want \" !!!
#to_extensions="${to_extensions}+smart" # COMBAK not needed when tidy is used

### BLOCKQUOTE

to_extensions="${to_extensions}+blank_before_blockquote"
# TODO mdview squashes blockquotes



### CODE BLOCKS ###
# Code blocks correspond to HTML <pre>.  Mdview can render both traditional,
# indented code block syntax, and backtick syntax.  It ignores arguments on
# the backtick line, except for "gettext", which enables mdview's translation
# feature inside the codeblock. Without "gettext" mdview doesn't translate the
# text in the code block. It would be tempting to enable +backtick_code_blocks
# but doing so would limit the editor's options (see below). Therefore we handle
# codeblocks with filter pandoc_codeblock.lua.

# +backtick_code_blocks ::
# Don't add +backtick_code_blocks unless you want ALL code blocks to render with
# backticks. Instead with this off and filter pandoc_codeblock.lua, the editor
# can choose between backticks - by not adding a class to <pre> ("mdview-nopot"
# and "mdview-gettext" don't count) - and indenting with spaces - by adding a
# class <pre>.

if [ $mdview_enable_gettext_support -eq 1 ]; then
export mdview_enable_gettext_support
# NOT! to_extensions= "${to_extensions}+backtick_code_blocks"   # case 2
lua_filters="${lua_filters} pandoc_codeblock.lua"               # case 2
fi



### LISTS ###
# Itemized lists (unordered bullets, ordered numbers) are problematic for
# several reasons, among which the following.
# 1.1. Mdview supports up to 2 levels. Pandoc supports unlimited levels of nesting.
#  ==> Mdview's limitation is too large to pursue a software work-around. Therefore
#    Content editors must restrict their production to maximum 2-level lists.
#    Function list_item_level_gt in check_for_mdview.sh enforces this requirement.
#
# 1.2. Mdview It doesn't support numbered lists; it squashes all numbered items into a paragraph.
#  ==> Case 2 in html5_to_ihtml.gawk implements a work-around for this issue.
#
# 2. Mdview's and pandoc's indentation rules don't play well together.
#  * Mdview's input: level 1 can be indented 0 or 1 space; an item can wrap and the
#    wrapped content can start with 0, 1, 2 and 3 spaces but not 4 because 4
#    triggers code block. Unfortunately, Mdview only supports 1-level of bullets.
#  * Pandoc's output: indentation is NOT flexible; level 1 can be indented 0-3
#    spaces; an item can wrap and content is wrapped with so-called "4-space"
#    rule (not configurable); 2nd level items must start with 4 spaces;
#    output item's content begins in column 9, and the item line wraps with
#    with 8 spaces. See https://github.com/jgm/pandoc/issues/2575 and 2210.
# ==> The only solution I found consists of
# A. running pandoc with option --wrap=none - to keep it from wrapping list items
# B. change faux bullets for level >= 2 item's bullets in ihtml_to_md.gawk
#
# 3. Mdview doesn't support ordered lists at all.
# ==> For this I added a work-around:
# C. insert bold item counter in item's content and replace containing <ol> with <ul>
#    This work-around is implemented in html5_to_ihtml.gawk, and works well with
#    work-around "B" for ordered lists.
#
# 4. Pandoc indents fenced code block _marks_ if the code block is part of a bullet item.
#    Consequently, mdview 2023.04 renders the indented marks instead of hiding them.
# ==> For this I added a work-around:
# D. Deindent fenced code block marks in ihtml_to_md.gawk.
#
# 5. Solution A isn't sufficient to always prevent list item wrapping. If the
#    item includes a hard break <br> then it will wrap at the hard break, again
#    with 4+ spaces, which make mdview enter block code mode.
# ==> For this I added a work-around:
# E.  Deindent hard-wrapped list items (just the hard wrapped part, not the
#     bullet line). The implementation passes hints across
#     html5_to_ihtml.gawk -> ihtml_to_md_pre.gawk -> ihtml_to_md.gawk.
#
# SUMMARY
# By combining all work-arounds we can output mdview-compatible markdown with
# up to 8 levels of mixed ordered <ol> and unordered <ul> bullet lists.
# Bullet items can include hard breaks <br>.
# Level-1 list items start in column 5 and neatly wrap if necessary (mdview
# wraps them). Level-2 and higher items start after level 5 with indentation
# increasing according to their level. However they can't neatly wrap
# because mdview doesn't support level 2 and higher. These items wrap at
# column 0 with intervening newlines, much like in a text editor.




### TABLES ###
# The cells of HTML tables may contain arbitrary block elements
# (multiple paragraphs, code blocks, lists, etc.). We follow a
# two-pronged approach. For simple tables, that is, tables without
# block elements - enable +pipe_tables.  For complex tables enable
# +grid_tables but unwrap the table into a list of markdown text blocks
# (see pandoc_table.lua) because mdview can't render grid tables, and
# also under the assumption that the HTML document used the table only
# to lay out the document (ex. Networking.html).

# +pipe_tables ::
# Pipe tables don't allow block-level content inside the cells, so if the
# table cells are not all "Plain" or "Para" pandoc falls back to HTML
# tables. GFM accepts pipe-table syntax. Mdview can render pipe tables with
# some limitations.
# Since we need line wrapping disabled, pipe table rows don't wrap.

# +grid_tables ::
# The cells of grid tables may contain arbitrary block elements
# (multiple paragraphs, code blocks, lists, etc.).
# However, mdview doesn't understand grid-table syntax.

# ~~~~~~
# enable even though 900 faqs do not include any <table>
# ~~~~~~
[ $mdview_can_render_pipe_table -gt 0 ] &&
to_extensions="${to_extensions}+pipe_tables"                   # case 3

# ~~~~~~
if false; then
# ~~~~~~
# Attempt to support complex tables - not working FIXME
to_extensions="${to_extensions}+grid_tables"                   # case 4
export mdview_can_render_pipe_table
lua_filters="${lua_filters} pandoc_table.lua"                  # case 4
# ~~~~~~
fi
# ~~~~~~

### HEADINGS
# Replace bold text for levels > $mdview_max_atx_level.
# Apply mdview-specific hack to insert a line before headings level 1 and 2.

to_extensions="${to_extensions}+blank_before_header"
export mdview_max_atx_level
export mdview_insert_line_before_atx12
export mdview_can_jump_to_anchor
lua_filters="${lua_filters} pandoc_header.lua"



### INLINES ###

export mdview_enable_gettext_support
lua_filters="${lua_filters} pandoc_inline.lua"

# ------------------------------------------------------------------------}}}



### BLOCKS ###
# Mdview automatically applies gettext translation files, if installed
# for the current language, to the matching markdown paragraph text.
# Editors can disable translation by wrapping the text between "%%nopot
# no" and "%%nopot yes". See /usr/share/mdview/examples/gettext.md.


# ~~~~~~
# 900: mdview translations are supported but %%nopot (only) isn't.
# ~~~~~~
export mdview_enable_gettext_support
lua_filters="${lua_filters} pandoc_block.lua"               # case 1

# ------------------------------------------------------------------------}}}




finalize_lua_filter_path() {
	local p f
	set -- $lua_filters
	for p; do f="$f -L ${0%/*}/$p"; done
	lua_filters="$f"
}

export LUA_PATH="${0%/*}/?.lua;;"
finalize_lua_filter_path

# pre-check pandoc command
pandoc $pandoc_options $lua_filters \
	--from html$from_extensions \
	--to   $mdview_markdown$to_extensions \
	< /dev/null > /dev/null

[ $? -eq 0 ]

